function [t, y] = solve_euler1d(f, tBereich, y0, h)
  // Integration der eindimensionalen DGL
  //    y' = f(y, t)
  // mit der Eulerformel.
  //   Argumente
  //       Funktion f
  //       Zeitintervall [tStart tEnd]
  //       Startwert y(0) = y0
  //       Schrittweite h
  //   Ergebnis
  //     Spaltenvektor t mit Zeitwerten
  //     Spaltenvektor y mit Naeherungsloesung y(t)
  t0 = tBereich(1);          // Anfangszeit
  t1 = tBereich(2);          // Endzeit
  N = ceil((t1 - t0)/h) + 1; // Zahl der Zeitpunkte
  y = zeros(N,1);            // Vektoren fuer die Loesung
  t = zeros(N,1);
  y(1) = y0;
  t(1) = t0;
  for I = 2:N
    y(I) = y(I-1) + h*f(t(I-1), y(I-1));
    t(I) = t(I-1) + h;
  end
endfunction

function [t, y] = solve_euler(f, tBereich, y0, h)
  // Integration der DGL
  //    y' = f(y, t)
  // mit der Eulerformel.
  //   Argumente
  //       Funktion f
  //       Zeitintervall [tStart tEnd]
  //       Startwerte y(0) = y0 (Vektor!)
  //       Schrittweite h
  //   Ergebnis
  //     Spaltenvektor t mit Zeitwerten
  //     Matrix y mit Spaltenvektoren y(t,i) mit Naeherungsloesung yi(t)
  //         der i-ten Komponente
  dim = size(y0, 2);         // Dimension von y (und von f!)
  t0 = tBereich(1);          // Anfangszeit
  t1 = tBereich(2);          // Endzeit
  N = ceil((t1 - t0)/h) + 1; // Zahl der Zeitpunkte
  y = zeros(N,dim);          // Vektoren fuer die Loesung
  t = zeros(N,1);
  y(1,:) = y0;
  t(1) = t0;
  for I = 2:N
    y(I,:) = y(I-1,:) + h*f(t(I-1), y(I-1,:))';
    t(I) = t(I-1) + h;
  end
endfunction

function [t, y] = solve_heun(f, tBereich, y0, h)
  // Integration der DGL
  //    y' = f(y, t)
  // mit dem Heun-Verfahren.
  //   Argumente
  //       Funktion f
  //       Zeitintervall [tStart tEnd]
  //       Startwerte y(0) = y0 (Vektor!)
  //       Schrittweite h
  //   Ergebnis
  //     Spaltenvektor t mit Zeitwerten
  //     Matrix y mit Spaltenvektoren y(t,i) mit Naeherungsloesung yi(t)
  //         der i-ten Komponente
  dim = size(y0, 2);         // Dimension von y (und von f!)
  t0 = tBereich(1);          // Anfangszeit
  t1 = tBereich(2);          // Endzeit
  N = ceil((t1 - t0)/h) + 1; // Zahl der Zeitpunkte
  y = zeros(N,dim);          // Vektoren fuer die Loesung
  t = zeros(N,1);
  y(1,:) = y0;
  t(1) = t0;
  for I = 2:N
    k1 = f(t(I-1), y(I-1,:))';
    k2 = f(t(I-1) + h, y(I-1,:) + h*k1)';
    y(I,:) = y(I-1,:) + 0.5*h*(k1 + k2);
    t(I) = t(I-1) + h;
  end
endfunction

function [t, y] = solve_heunadapt(f, tBereich, y0, tol, info)
  // Integration der DGL
  //    y' = f(y, t)
  // mit dem Heun-Verfahren mit adaptiver Schrittweite
  //   Argumente
  //       Funktion f
  //       Zeitintervall [tStart tEnd]
  //       Startwerte y(0) = y0 (Vektor!)
  //       lokale Genauigkeit tol
  //       optional: info = true -> Kontrollausgabe   
  //   Ergebnis
  //     Spaltenvektor t mit Zeitwerten
  //     Matrix y mit Spaltenvektoren y(t,i) mit Naeherungsloesung yi(t)
  //         der i-ten Komponente
  //

  if argn(2) == 4
    info = %F;
  end

  dim = length(y0, 2);           // Dimension von y (und von f!)
  t0 = tBereich(1);
  tEnd = tBereich(2);
  
  y = zeros(1,dim);         // Arrays fuer die Loesung
  t = zeros(1,1);
  y(1,:) = y0;              // Anfangswerte eintragen
  t(1,1) = t0;
  h = tEnd - t0;     // probieren wir es mal mit einem Schritt
  
  while (t0 < tEnd)
    // Schritt mit h und 2 * h/2
    showStep('Rechnen', h, info)
    y1    = heunstep(f, t0, y0, h);
    yhalb = heunstep(f, t0, y0, h/2);
    y2    = heunstep(f, t0 + h/2, yhalb, h/2);
    epsrel = max(abs(y2 - y1)./y2);

    while (epsrel > tol)    // zu ungenau -> verfeinern
      h = h/2;
      y1    = heunstep(f, t0, y0, h);
      yhalb = heunstep(f, t0, y0, h/2);
      y2    = heunstep(f, t0 + h/2, yhalb, h/2);
      epsrel = max(abs(y2 - y1)./y2);
      showStep('Verfeinern', h, info)
    end
      
    // ok, h ist klein genug
    y0 = y2;
    t0 = t0 + h;
    y($+1,:) = y0;
    t($+1,1) = t0;
          
    // evtl. h vergroessern
     if (epsrel < tol/2)
        h = 2*h;
        showStep('Vergroebern', h, info)
    end
  end
endfunction

function yneu = heunstep(f, t0, y0, h)
  // ein Schritt mit dem Heun-Verfahren
  k1 = f(t0, y0)';
  k2 = f(t0 + h, y0 + h*k1)';
  yneu = y0 + 0.5*h*(k1 + k2);
endfunction

function showStep(s, h, info)
  // gibt ggf. Schrittweite aus
  if info
    mprintf('%12s, h = %20.16f\n', s, h)
  end
endfunction
